package hatelace.instructions;

import hatelace.*;

public class Mul implements Instruction {
    private Address src1;
    private Address src2;
    private Address dest;

    /** Multiply contents of two addresses and store the result in a third memory address. */
    public Mul(Address src1, Address src2, Address dest) {
        this.src1 = src1;
        this.src2 = src2;
        this.dest = dest;
    }

    public void execute(Memory memory, ProgramCounter PC) {
        memory.write(this.dest, memory.read(this.src1).multiply(memory.read(this.src2)));
        PC.incPC();
    }

    public String opcode() {
        return "MUL";
    }

    public Object[] operands() {
        return new Object[] { this.src1, this.src2, this.dest };
    }

    public String toString() {
        return String.format("%s [%s] [%s] [%s]", this.opcode(), this.src1, this.src2, this.dest);
    }
}
