package hatelace.memtypes;

import hatelace.Address;
import hatelace.Memory;
import hatelace.Word;

public class LongMemory extends Memory {
    protected long[] memory;

    public LongMemory() {
        this.memory = new long[256];
    }

    public LongMemory(int size) {
        this.memory = new long[size];
    }

    public Word read(Address address) {
        return new LongWord(this.memory[address.getAddress()]);
    }

    public int size() {
        return this.memory.length;
    }

    public void write(Address address, Word data) {
        this.memory[address.getAddress()] = data.getValue();
    }

    /** Dump as IHEX-like format (see https://en.wikipedia.org/wiki/Longel_HEX) */
    public void dump() {
        for(int i = 0; i < this.memory.length; i+= 16) {
            System.out.printf(":%02X%04X00 ", 16, i);
            for(int j = 0; j < 16; j++) {
                long content = this.memory[i + j];

                // Print it as faded if it's zero
                if (content == 0)
                    System.out.print("\u001B[2m");

                System.out.printf("%02X ", this.memory[i + j]);
                System.out.print("\u001B[0m");
            }
            System.out.println();
        }
    }
}
